@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.1   2011/04/01
+ * Version: 1.8.2   2011/06/20
  */
 
 #include <linux/version.h>
@@ -33,30 +33,6 @@
 
 #include "internal.h"
 
-#ifndef CONFIG_CCSECURITY_OMIT_USERSPACE_LOADER
-
-/* Path to the policy loader. (default = CONFIG_CCSECURITY_DEFAULT_LOADER) */
-static const char *ccs_loader;
-
-#if 0
-/**
- * ccs_loader_setup - Specify the policy loader to use.
- *
- * @str: Path to the policy loader.
- *
- * Returns 0.
- */
-static int __init ccs_loader_setup(char *str)
-{
-	ccs_loader = str;
-	return 0;
-}
-
-__setup("CCS_loader=", ccs_loader_setup);
-#endif
-
-#endif
-
 #if 0
 /**
  * ccs_setup - Set enable/disable upon boot.
@@ -79,6 +55,26 @@ __setup("ccsecurity=", ccs_setup);
 
 #ifndef CONFIG_CCSECURITY_OMIT_USERSPACE_LOADER
 
+/* Path to the policy loader. (default = CONFIG_CCSECURITY_POLICY_LOADER) */
+static const char *ccs_loader;
+
+#if 0
+/**
+ * ccs_loader_setup - Set policy loader.
+ *
+ * @str: Program to use as a policy loader (e.g. /sbin/ccs-init ).
+ *
+ * Returns 0.
+ */
+static int __init ccs_loader_setup(char *str)
+{
+	ccs_loader = str;
+	return 0;
+}
+
+__setup("CCS_loader=", ccs_loader_setup);
+#endif
+
 /**
  * ccs_policy_loader_exists - Check whether /sbin/ccs-init exists.
  *
@@ -86,17 +82,10 @@ __setup("ccsecurity=", ccs_setup);
  */
 static _Bool ccs_policy_loader_exists(void)
 {
-	/*
-	 * Don't activate MAC if the path given by 'CCS_loader=' option doesn't
-	 * exist. If the initrd includes /sbin/init but real-root-dev has not
-	 * mounted on / yet, activating MAC will block the system since
-	 * policies are not loaded yet.
-	 * Thus, let do_execve() call this function everytime.
-	 */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 28)
 	struct path path;
 	if (!ccs_loader)
-		ccs_loader = CONFIG_CCSECURITY_DEFAULT_LOADER;
+		ccs_loader = CONFIG_CCSECURITY_POLICY_LOADER;
 	if (kern_path(ccs_loader, LOOKUP_FOLLOW | LOOKUP_POSITIVE,
 		      &path) == 0) {
 		path_put(&path);
@@ -105,7 +94,7 @@ static _Bool ccs_policy_loader_exists(void)
 #else
 	struct nameidata nd;
 	if (!ccs_loader)
-		ccs_loader = CONFIG_CCSECURITY_DEFAULT_LOADER;
+		ccs_loader = CONFIG_CCSECURITY_POLICY_LOADER;
 	if (path_lookup(ccs_loader, LOOKUP_FOLLOW | LOOKUP_POSITIVE,
 			&nd) == 0) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
@@ -146,6 +135,26 @@ static int ccs_run_loader(void *unused)
 
 #endif
 
+/* Path to the trigger. (default = CONFIG_CCSECURITY_ACTIVATION_TRIGGER) */
+static const char *ccs_trigger;
+
+#if 0
+/**
+ * ccs_trigger_setup - Set trigger for activation.
+ *
+ * @str: Program to use as an activation trigger (e.g. /sbin/init ).
+ *
+ * Returns 0.
+ */
+static int __init ccs_trigger_setup(char *str)
+{
+	ccs_trigger = str;
+	return 0;
+}
+
+__setup("CCS_trigger=", ccs_trigger_setup);
+#endif
+
 /**
  * ccs_load_policy - Run external policy loader to load policy.
  *
@@ -161,13 +170,16 @@ static int ccs_run_loader(void *unused)
  */
 void ccs_load_policy(const char *filename)
 {
-	if (ccsecurity_ops.disabled)
+	static _Bool done;
+	if (ccsecurity_ops.disabled || done)
 		return;
-	if (strcmp(filename, "/sbin/init") &&
-	    strcmp(filename, CONFIG_CCSECURITY_ALTERNATIVE_TRIGGER))
+	if (!ccs_trigger)
+		ccs_trigger = CONFIG_CCSECURITY_ACTIVATION_TRIGGER;
+	if (strcmp(filename, ccs_trigger))
 		return;
 	if (!ccs_policy_loader_exists())
 		return;
+	done = 1;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 	{
 		char *argv[2];
