@@ -1,20 +1,15 @@
 /*
  * lsm.c
  *
- * Copyright (C) 2010  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
+ * Copyright (C) 2010-2011  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
  *
- * Version: 1.0.5   2010/11/22
+ * Version: 1.0.15   2011/06/20
  */
+
 #include "internal.h"
 #include <linux/security.h>
 #include <linux/namei.h>
 
-/*
- * "struct cred" exists in 2.6.29 and later kernels.
- * But this declaration is harmless for 2.6.28 and earlier kernels.
- */
-struct cred;
-
 /* Prototype definition. */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
 static void ccs_task_security_gc(void);
@@ -25,7 +20,12 @@ static void (*ccs___put_task_struct) (struct task_struct *t);
 #endif
 
 /* Dummy security context for avoiding NULL pointer dereference. */
-static struct ccs_security ccs_null_security = {
+static struct ccs_security ccs_oom_security = {
+	.ccs_domain_info = &ccs_kernel_domain
+};
+
+/* Dummy security context for avoiding NULL pointer dereference. */
+static struct ccs_security ccs_default_security = {
 	.ccs_domain_info = &ccs_kernel_domain
 };
 
@@ -150,7 +150,7 @@ static void ccs_rcu_free(void *arg)
 static void ccs_del_security(struct ccs_security *ptr)
 {
 	unsigned long flags;
-	if (ptr == &ccs_null_security)
+	if (ptr == &ccs_default_security || ptr == &ccs_oom_security)
 		return;
 	spin_lock_irqsave(&ccs_task_security_list_lock, flags);
 	list_del_rcu(&ptr->list);
@@ -173,7 +173,7 @@ static void ccs_del_security(struct ccs_security *ptr)
 static void ccs_clear_execve(int ret, struct ccs_security *security)
 {
 	struct ccs_execve *ee;
-	if (security == &ccs_null_security)
+	if (security == &ccs_default_security || security == &ccs_oom_security)
 		return;
 	ee = security->ee;
 	security->ee = NULL;
@@ -341,8 +341,10 @@ static void ccs_cred_transfer(struct cred *new, const struct cred *old)
 	original_security_ops.cred_transfer(new, old);
 	new_security = ccs_find_cred_security(new);
 	old_security = ccs_find_cred_security(old);
-	if (new_security == &ccs_null_security ||
-	    old_security == &ccs_null_security)
+	if (new_security == &ccs_default_security ||
+	    new_security == &ccs_oom_security ||
+	    old_security == &ccs_default_security ||
+	    old_security == &ccs_oom_security)
 		return;
 	new_security->ccs_flags = old_security->ccs_flags;
 	new_security->ccs_domain_info = old_security->ccs_domain_info;
@@ -422,7 +424,7 @@ static void ccs_bprm_free_security(struct linux_binprm *bprm)
 	 * If do_execve() succeeded,
 	 * ccs_clear_execve(0, ccs_current_security());
 	 * is called before calling below one.
-	 * Thus, below call becomes no-op if do_execve() succeeded. 
+	 * Thus, below call becomes no-op if do_execve() succeeded.
 	 */
 	ccs_clear_execve(-1, ccs_current_security());
 }
@@ -478,7 +480,8 @@ static void ccs_bprm_committing_creds(struct linux_binprm *bprm)
 	while (!original_security_ops.bprm_committing_creds);
 	original_security_ops.bprm_committing_creds(bprm);
 	old_security = ccs_current_security();
-	if (old_security == &ccs_null_security)
+	if (old_security == &ccs_default_security ||
+	    old_security == &ccs_oom_security)
 		return;
 	ccs_clear_execve(0, old_security);
 	/* Update current task's cred's domain for future fork(). */
@@ -498,11 +501,11 @@ static void ccs_bprm_committing_creds(struct linux_binprm *bprm)
  */
 static int ccs_bprm_check_security(struct linux_binprm *bprm)
 {
-	int rc;
 	struct ccs_security *security = ccs_current_security();
-	if (security == &ccs_null_security)
+	if (security == &ccs_default_security || security == &ccs_oom_security)
 		return -ENOMEM;
-	if (!security->cred) {
+	if (!security->ee) {
+		int rc;
 		if (!ccs_policy_loaded)
 			ccs_load_policy(bprm->filename);
 		rc = ccs_start_execve(bprm, &security->ee);
@@ -517,17 +520,9 @@ static int ccs_bprm_check_security(struct linux_binprm *bprm)
 			security->cred = bprm->cred;
 #endif
 		}
-	} else {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
-		rc = ccs_open_permission(bprm->file);
-#else
-		/* 01 means "read". */
-		rc = ccs_open_permission(bprm->file->f_dentry,
-					 bprm->file->f_vfsmnt, 01);
-#endif
+		if (rc)
+			return rc;
 	}
-	if (rc)
-		return rc;
 	while (!original_security_ops.bprm_check_security);
 	return original_security_ops.bprm_check_security(bprm);
 }
@@ -543,13 +538,11 @@ static int ccs_bprm_check_security(struct linux_binprm *bprm)
  */
 static int ccs_open(struct file *f)
 {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 30)
-	/* Don't check read permission here if called from do_execve(). */
-	if (current->in_execve)
-		return 0;
-#endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
 	return ccs_open_permission(f);
+#elif defined(RHEL_MAJOR) && RHEL_MAJOR == 6
+	return ccs_open_permission(f->f_path.dentry, f->f_path.mnt,
+				   f->f_flags);
 #else
 	return ccs_open_permission(f->f_path.dentry, f->f_path.mnt,
 				   f->f_flags + 1);
@@ -611,6 +604,10 @@ static int ccs_open(struct inode *inode, int mask, struct nameidata *nd)
 	int flags;
 	if (!nd || !nd->dentry)
 		return 0;
+	/* open_exec() passes MAY_EXEC . */
+	if (mask == MAY_EXEC && inode && S_ISREG(inode->i_mode) &&
+	    (ccs_current_flags() & CCS_TASK_IS_IN_EXECVE))
+		mask = MAY_READ;
 	/*
 	 * This flags value is passed to ACC_MODE().
 	 * ccs_open_permission() for older versions uses old ACC_MODE().
@@ -855,8 +852,7 @@ static int ccs_inode_getattr(struct vfsmount *mnt, struct dentry *dentry)
 static int ccs_path_mknod(struct path *dir, struct dentry *dentry, int mode,
 			  unsigned int dev)
 {
-	int rc = ccs_mknod_permission(dir->dentry->d_inode, dentry, dir->mnt,
-				      mode, dev);
+	int rc = ccs_mknod_permission(dentry, dir->mnt, mode, dev);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_mknod);
@@ -874,8 +870,7 @@ static int ccs_path_mknod(struct path *dir, struct dentry *dentry, int mode,
  */
 static int ccs_path_mkdir(struct path *dir, struct dentry *dentry, int mode)
 {
-	int rc = ccs_mkdir_permission(dir->dentry->d_inode, dentry, dir->mnt,
-				      mode);
+	int rc = ccs_mkdir_permission(dentry, dir->mnt, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_mkdir);
@@ -892,7 +887,7 @@ static int ccs_path_mkdir(struct path *dir, struct dentry *dentry, int mode)
  */
 static int ccs_path_rmdir(struct path *dir, struct dentry *dentry)
 {
-	int rc = ccs_rmdir_permission(dir->dentry->d_inode, dentry, dir->mnt);
+	int rc = ccs_rmdir_permission(dentry, dir->mnt);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_rmdir);
@@ -909,7 +904,7 @@ static int ccs_path_rmdir(struct path *dir, struct dentry *dentry)
  */
 static int ccs_path_unlink(struct path *dir, struct dentry *dentry)
 {
-	int rc = ccs_unlink_permission(dir->dentry->d_inode, dentry, dir->mnt);
+	int rc = ccs_unlink_permission(dentry, dir->mnt);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_unlink);
@@ -928,8 +923,7 @@ static int ccs_path_unlink(struct path *dir, struct dentry *dentry)
 static int ccs_path_symlink(struct path *dir, struct dentry *dentry,
 			    const char *old_name)
 {
-	int rc = ccs_symlink_permission(dir->dentry->d_inode, dentry, dir->mnt,
-					old_name);
+	int rc = ccs_symlink_permission(dentry, dir->mnt, old_name);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_symlink);
@@ -949,9 +943,7 @@ static int ccs_path_symlink(struct path *dir, struct dentry *dentry,
 static int ccs_path_rename(struct path *old_dir, struct dentry *old_dentry,
 			   struct path *new_dir, struct dentry *new_dentry)
 {
-	int rc = ccs_rename_permission(old_dir->dentry->d_inode, old_dentry,
-				       new_dir->dentry->d_inode, new_dentry,
-				       old_dir->mnt);
+	int rc = ccs_rename_permission(old_dentry, new_dentry, old_dir->mnt);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_rename);
@@ -971,8 +963,7 @@ static int ccs_path_rename(struct path *old_dir, struct dentry *old_dentry,
 static int ccs_path_link(struct dentry *old_dentry, struct path *new_dir,
 			 struct dentry *new_dentry)
 {
-	int rc = ccs_link_permission(old_dentry, new_dir->dentry->d_inode,
-				     new_dentry, new_dir->mnt);
+	int rc = ccs_link_permission(old_dentry, new_dentry, new_dir->mnt);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_link);
@@ -996,7 +987,7 @@ static int ccs_path_link(struct dentry *old_dentry, struct path *new_dir,
 static int ccs_inode_mknod(struct inode *dir, struct dentry *dentry,
 			   struct vfsmount *mnt, int mode, dev_t dev)
 {
-	int rc = ccs_mknod_permission(dir, dentry, mnt, mode, dev);
+	int rc = ccs_mknod_permission(dentry, mnt, mode, dev);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mknod);
@@ -1016,7 +1007,7 @@ static int ccs_inode_mknod(struct inode *dir, struct dentry *dentry,
 static int ccs_inode_mkdir(struct inode *dir, struct dentry *dentry,
 			   struct vfsmount *mnt, int mode)
 {
-	int rc = ccs_mkdir_permission(dir, dentry, mnt, mode);
+	int rc = ccs_mkdir_permission(dentry, mnt, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mkdir);
@@ -1035,7 +1026,7 @@ static int ccs_inode_mkdir(struct inode *dir, struct dentry *dentry,
 static int ccs_inode_rmdir(struct inode *dir, struct dentry *dentry,
 			   struct vfsmount *mnt)
 {
-	int rc = ccs_rmdir_permission(dir, dentry, mnt);
+	int rc = ccs_rmdir_permission(dentry, mnt);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_rmdir);
@@ -1054,7 +1045,7 @@ static int ccs_inode_rmdir(struct inode *dir, struct dentry *dentry,
 static int ccs_inode_unlink(struct inode *dir, struct dentry *dentry,
 			    struct vfsmount *mnt)
 {
-	int rc = ccs_unlink_permission(dir, dentry, mnt);
+	int rc = ccs_unlink_permission(dentry, mnt);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_unlink);
@@ -1074,7 +1065,7 @@ static int ccs_inode_unlink(struct inode *dir, struct dentry *dentry,
 static int ccs_inode_symlink(struct inode *dir, struct dentry *dentry,
 			     struct vfsmount *mnt, const char *old_name)
 {
-	int rc = ccs_symlink_permission(dir, dentry, mnt, old_name);
+	int rc = ccs_symlink_permission(dentry, mnt, old_name);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_symlink);
@@ -1098,8 +1089,7 @@ static int ccs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
 			    struct dentry *new_dentry,
 			    struct vfsmount *new_mnt)
 {
-	int rc = ccs_rename_permission(old_dir, old_dentry, new_dir,
-				       new_dentry, new_mnt);
+	int rc = ccs_rename_permission(old_dentry, new_dentry, new_mnt);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_rename);
@@ -1123,7 +1113,7 @@ static int ccs_inode_link(struct dentry *old_dentry, struct vfsmount *old_mnt,
 			  struct inode *dir, struct dentry *new_dentry,
 			  struct vfsmount *new_mnt)
 {
-	int rc = ccs_link_permission(old_dentry, dir, new_dentry, new_mnt);
+	int rc = ccs_link_permission(old_dentry, new_dentry, new_mnt);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_link);
@@ -1144,7 +1134,7 @@ static int ccs_inode_link(struct dentry *old_dentry, struct vfsmount *old_mnt,
 static int ccs_inode_create(struct inode *dir, struct dentry *dentry,
 			    struct vfsmount *mnt, int mode)
 {
-	int rc = ccs_mknod_permission(dir, dentry, mnt, mode, 0);
+	int rc = ccs_mknod_permission(dentry, mnt, mode, 0);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_create);
@@ -1166,7 +1156,7 @@ static int ccs_inode_create(struct inode *dir, struct dentry *dentry,
 static int ccs_inode_mknod(struct inode *dir, struct dentry *dentry, int mode,
 			   dev_t dev)
 {
-	int rc = ccs_mknod_permission(dir, dentry, NULL, mode, dev);
+	int rc = ccs_mknod_permission(dentry, NULL, mode, dev);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mknod);
@@ -1184,7 +1174,7 @@ static int ccs_inode_mknod(struct inode *dir, struct dentry *dentry, int mode,
  */
 static int ccs_inode_mkdir(struct inode *dir, struct dentry *dentry, int mode)
 {
-	int rc = ccs_mkdir_permission(dir, dentry, NULL, mode);
+	int rc = ccs_mkdir_permission(dentry, NULL, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mkdir);
@@ -1201,7 +1191,7 @@ static int ccs_inode_mkdir(struct inode *dir, struct dentry *dentry, int mode)
  */
 static int ccs_inode_rmdir(struct inode *dir, struct dentry *dentry)
 {
-	int rc = ccs_rmdir_permission(dir, dentry, NULL);
+	int rc = ccs_rmdir_permission(dentry, NULL);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_rmdir);
@@ -1218,7 +1208,7 @@ static int ccs_inode_rmdir(struct inode *dir, struct dentry *dentry)
  */
 static int ccs_inode_unlink(struct inode *dir, struct dentry *dentry)
 {
-	int rc = ccs_unlink_permission(dir, dentry, NULL);
+	int rc = ccs_unlink_permission(dentry, NULL);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_unlink);
@@ -1237,7 +1227,7 @@ static int ccs_inode_unlink(struct inode *dir, struct dentry *dentry)
 static int ccs_inode_symlink(struct inode *dir, struct dentry *dentry,
 			     const char *old_name)
 {
-	int rc = ccs_symlink_permission(dir, dentry, NULL, old_name);
+	int rc = ccs_symlink_permission(dentry, NULL, old_name);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_symlink);
@@ -1257,8 +1247,7 @@ static int ccs_inode_symlink(struct inode *dir, struct dentry *dentry,
 static int ccs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
 			    struct inode *new_dir, struct dentry *new_dentry)
 {
-	int rc = ccs_rename_permission(old_dir, old_dentry, new_dir,
-				       new_dentry, NULL);
+	int rc = ccs_rename_permission(old_dentry, new_dentry, NULL);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_rename);
@@ -1278,7 +1267,7 @@ static int ccs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
 static int ccs_inode_link(struct dentry *old_dentry, struct inode *dir,
 			  struct dentry *new_dentry)
 {
-	int rc = ccs_link_permission(old_dentry, dir, new_dentry, NULL);
+	int rc = ccs_link_permission(old_dentry, new_dentry, NULL);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_link);
@@ -1297,7 +1286,7 @@ static int ccs_inode_link(struct dentry *old_dentry, struct inode *dir,
 static int ccs_inode_create(struct inode *dir, struct dentry *dentry,
 			    int mode)
 {
-	int rc = ccs_mknod_permission(dir, dentry, NULL, mode, 0);
+	int rc = ccs_mknod_permission(dentry, NULL, mode, 0);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_create);
@@ -1892,7 +1881,7 @@ static int ccs_file_ioctl(struct file *filp, unsigned int cmd,
  * ccs_prepend - Copy of prepend() in fs/dcache.c.
  *
  * @buffer: Pointer to "struct char *".
- * @buflen: Pointer to int which holds size of @buffer. 
+ * @buflen: Pointer to int which holds size of @buffer.
  * @str:    String to copy.
  *
  * Returns 0 on success, negative value otherwise.
@@ -1982,7 +1971,7 @@ out:
 
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
 
 #include <linux/mount.h>
 #include <linux/fs_struct.h>
@@ -2025,7 +2014,7 @@ static int __init ccs_kernel_read(struct file *file, unsigned long offset,
  *
  * @keyline: Function to find.
  *
- * Returns address if specified function on success, NULL otherwise.
+ * Returns address of specified function on success, NULL otherwise.
  */
 static void *__init ccs_find_symbol(const char *keyline)
 {
@@ -2125,7 +2114,7 @@ static void * __init ccs_find_variable(void *function, unsigned long addr,
 	int i;
 	u8 *base;
 	u8 *cp = function;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
 	if (*symbol == ' ')
 		base = ccs_find_symbol(symbol);
 	else
@@ -2289,13 +2278,50 @@ out:
 #endif
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 15)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
 
 /* Never mark this variable as __initdata . */
 static spinlock_t ccs_vfsmount_lock;
 
 /**
- * lsm_floup - Dummy function which does identical to follow_up() in fs/namei.c.
+ * lsm_umnt_tr - Dummy function which prototype is identical to umount_tree() in fs/namespace.c.
+ *
+ * @mnt: Pointer to "struct vfsmount".
+ *
+ * Returns nothing.
+ */
+static void lsm_umnt_tr(struct vfsmount *mnt)
+{
+	/* Nothing to do because this function is not called. */
+}
+
+/**
+ * lsm__put_nmspce - Dummy function which does identical to __put_namespace() in fs/namespace.c.
+ *
+ * @namespace: Pointer to "struct namespace".
+ *
+ * Returns nothing.
+ *
+ * Never mark this function as __init in order to make sure that compiler
+ * generates identical code for __put_namespace() and this function.
+ */
+static void lsm__put_nmspce(struct namespace *namespace)
+{
+	down_write(&namespace->sem);
+	spin_lock(&ccs_vfsmount_lock);
+	lsm_umnt_tr(namespace->root);
+	spin_unlock(&ccs_vfsmount_lock);
+	up_write(&namespace->sem);
+	kfree(namespace);
+}
+
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 15)
+
+/* Never mark this variable as __initdata . */
+static spinlock_t ccs_vfsmount_lock;
+
+/**
+ * lsm_flwup - Dummy function which does identical to follow_up() in fs/namei.c.
  *
  * @mnt:    Pointer to "struct vfsmount *".
  * @dentry: Pointer to "struct dentry *".
@@ -2356,7 +2382,36 @@ static void lsm_pin(struct vfsmount *mnt)
  */
 static bool __init ccs_find_vfsmount_lock(void)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 15)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
+	void *cp;
+	spinlock_t *ptr;
+	/* Guess "spinlock_t vfsmount_lock;". */
+	cp = ccs_find_variable(lsm__put_nmspce,
+			       (unsigned long) &ccs_vfsmount_lock,
+			       " __put_namespace\n");
+	if (!cp) {
+		printk(KERN_ERR "Can't resolve __put_namespace().\n");
+		goto out;
+	}
+	/* This should be "spinlock_t *vfsmount_lock;". */
+	ptr = *(spinlock_t **) cp;
+	if (!ptr) {
+		printk(KERN_ERR "Can't resolve vfsmount_lock .\n");
+		goto out;
+	}
+	ccsecurity_exports.vfsmount_lock = ptr;
+	printk(KERN_INFO "vfsmount_lock=%p\n", ptr);
+	return true;
+out:
+	/*
+	 * Dummy call for preventing lsm_umnt_tr() from being inlined into
+	 * lsm__put_nmspce().
+	 */
+	cp = ccs_find_variable(lsm_umnt_tr,
+			       (unsigned long) &ccs_vfsmount_lock,
+			       " __put_namespace\n");
+	return false;
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 15)
 	void *cp;
 	spinlock_t *ptr;
 	/* Guess "spinlock_t vfsmount_lock;". */
@@ -2529,7 +2584,7 @@ static int __init ccs_init(void)
 #endif
 	ccs_main_init();
 	ccs_update_security_ops(ops);
-	printk(KERN_INFO "AKARI: 1.0.5   2010/11/22\n");
+	printk(KERN_INFO "AKARI: 1.0.15   2011/06/20\n");
 	printk(KERN_INFO
 	       "Access Keeping And Regulating Instrument registered.\n");
 	return 0;
@@ -2571,8 +2626,8 @@ bool ccs_used_by_cred(const struct ccs_domain_info *domain)
  *
  * @task: Pointer to "struct task_struct".
  *
- * Returns pointer to "struct ccs_security" on success, &ccs_null_security
- * otherwise.
+ * Returns pointer to "struct ccs_security" on success, &ccs_oom_security on
+ * out of memory, &ccs_default_security otherwise.
  *
  * If @task is current thread and "struct ccs_security" for current thread was
  * not found, I try to allocate it. But if allocation failed, current thread
@@ -2645,8 +2700,8 @@ struct ccs_security *ccs_find_task_security(const struct task_struct *task)
 		 * associated with that thread's "struct cred".
 		 *
 		 * Since that snapshot will be used as initial data when that
-		 * thread allocates "struct ccs_security" for that thread,
-		 * we can return that snapshot rather than &ccs_null_security.
+		 * thread allocates "struct ccs_security" for that thread, we
+		 * can return that snapshot rather than &ccs_default_security.
 		 *
 		 * Since this function is called by only ccs_select_one() and
 		 * ccs_read_pid() (via ccs_task_domain() and ccs_task_flags()),
@@ -2657,7 +2712,7 @@ struct ccs_security *ccs_find_task_security(const struct task_struct *task)
 		 */
 		return ccs_find_cred_security(__task_cred(task));
 #else
-		return &ccs_null_security;
+		return &ccs_default_security;
 #endif
 	}
 	/* Use GFP_ATOMIC because caller may have called rcu_read_lock(). */
@@ -2666,17 +2721,19 @@ struct ccs_security *ccs_find_task_security(const struct task_struct *task)
 		printk(KERN_WARNING "Unable to allocate memory for pid=%u\n",
 		       task->pid);
 		send_sig(SIGKILL, current, 0);
-		return &ccs_null_security;
+		return &ccs_oom_security;
 	}
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
 	*ptr = *ccs_find_cred_security(task->cred);
 #else
-	*ptr = ccs_null_security;
+	*ptr = ccs_default_security;
 #endif
 	get_task_struct((struct task_struct *) task);
 	ptr->task = (struct task_struct *) task;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
 	/* ptr->cred may point to garbage. I need to explicitly clear. */
 	ptr->cred = NULL;
+#endif
 	ccs_add_task_security(ptr, list);
 	return ptr;
 }
@@ -2711,7 +2768,7 @@ static int ccs_copy_cred_security(const struct cred *new,
  *
  * @cred: Pointer to "struct cred".
  *
- * Returns pointer to "struct ccs_security" on success, &ccs_null_security
+ * Returns pointer to "struct ccs_security" on success, &ccs_default_security
  * otherwise.
  */
 static struct ccs_security *ccs_find_cred_security(const struct cred *cred)
@@ -2727,7 +2784,7 @@ static struct ccs_security *ccs_find_cred_security(const struct cred *cred)
 		return ptr;
 	}
 	rcu_read_unlock();
-	return &ccs_null_security;
+	return &ccs_default_security;
 }
 
 /**
@@ -2747,7 +2804,7 @@ static void ccs_task_security_gc(void)
 	unsigned int idx;
 	if (!mutex_trylock(&lock))
 		return;
-	/* Checking everytime is too wasteful. */
+	/* Checking every time is too wasteful. */
 	if (counter++ < 1024)
 		goto skip;
 	counter = 0;
